require 'pico_http_parser'

module Soba
  class Parser
    CRLF = "\r\n".freeze

    class InvalidRequest
    end

    attr_reader :socket

    def initialize(socket)
      @socket = socket
      @headers = nil
      @body = nil
    end

    def headers
      @headers ||= begin
        request_body = ""
        while (line = socket.readline) != CRLF
          request_body << line
        end
        request_body << CRLF
        debug request_body
        ret = PicoHTTPParser.parse_http_request(request_body, @headers ||= {})
        debug ret
        raise InvalidRequest, "ret: #{ret}" if ret < 0
        @headers
      end
    end

    alias env headers

    # http or https
    def request_schema
      @request_schema ||= server_protocol.split("/").first.downcase
    end

    def content_length
      headers["CONTENT_LENGTH"]&.to_i
    end

    def body
      @body ||= begin
        StringIO.new(content_length ? socket.read(content_length) : '').binmode
      end
    end

    %w{SERVER_PROTOCOL}.each do |header|
      define_method header.downcase do
        headers[header]
      end
    end

    private
    def debug(*args)
      STDERR.puts(args) if ENV["SOBA_DEBUG"] == '1'
    end
  end
end